﻿using ProjectAtelier.Entities;
using ProjectAtelier.Repositories;
using ProjectAtelier.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectAtelier.Forms
{
    public partial class FormProductMaterial : Form
    {
        private readonly IProductRepository _productRepository;
        private readonly IProductMaterialRepository _feedReplenishmentRepository;
        private int? _productId;
        public int Id
        {
            set
            {
                try
                {
                    var product = _productRepository.ReadProductById(value);
                    if (product == null)
                    {
                        throw new InvalidDataException(nameof(product));
                    }
                    textBoxName.Text = product.Name;
                    comboBoxProduct.SelectedItem = product.View;
                    numericUpDownCount.Value = product.CountMaterial;
                    _productId = value;

                    // Заполняем DataGridView данными из product.ProductMaterial
                    dataGridView.Rows.Clear();
                    foreach (var material in product.ProductMaterial)
                    {
                        dataGridView.Rows.Add(material.MaterialId, material.Count);
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
        }
        public FormProductMaterial(IProductMaterialRepository materialConsumptionRepository, IProductRepository productRepository, IMaterialRepository materialRepository)
        {
            InitializeComponent();
            _feedReplenishmentRepository = materialConsumptionRepository ?? throw new ArgumentNullException(nameof(materialConsumptionRepository));
            _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository));

            comboBoxProduct.DataSource = Enum.GetValues(typeof(ProductView));
           
        }
        private void buttonAdd_Click(object sender, EventArgs e)
        {
            if (dataGridView.RowCount < 1)
            {
                MessageBox.Show("Имеются незаполненные поля", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxName.Text) || comboBoxProduct.SelectedIndex < 0)
                {
                    throw new Exception("Имеются незаполненные поля");
                }
                if (_productId.HasValue)
                {
                    _productRepository.UpdateProduct(CreateProduct(_productId.Value));
                }
                else
                {
                    _productRepository.CreateProduct(CreateProduct(0));
                }
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void buttonCancel_Click(object sender, EventArgs e) => Close();
        private List<ProductMaterial> CreateListMaterialFromDataGrid()
        {
            var list = new List<ProductMaterial>();
            foreach (DataGridViewRow row in dataGridView.Rows)
            {
                if (row.Cells["ColumnMaterial"].Value == null || row.Cells["ColumnCount"].Value == null)
                {
                    continue;
                }
                list.Add(new ProductMaterial
                {
                    MaterialId = Convert.ToInt32(row.Cells["ColumnMaterial"].Value),
                    Count = Convert.ToInt32(row.Cells["ColumnCount"].Value)
                });
            }
            return list;
        }
        private Product CreateProduct(int id) => new Product
        {
            Id = id,
            Name = textBoxName.Text,
            View = (ProductView)comboBoxProduct.SelectedItem,
            CountMaterial = Convert.ToInt32(numericUpDownCount.Value),
            ProductMaterial = CreateListMaterialFromDataGrid()
        };
    }
}